<?php
namespace OrganizationsManagement\Service;
use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Factory;
use Teknisa\Model\Teknisa\Repositories;
use Teknisa\Model\Geral;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;

class Table
{
    /** @var Utilities **/
    protected $utilities;

    /** @var EntityManager **/
    protected $entityManager;

    /** @var TeknisaEnvironment **/
    protected $environment;

    /** @var \Teknisa\Model\Geral\Repositories\Tabela **/
    protected $repositoryTabela;
    
    /** @var \Teknisa\Model\Integracao\Repositories\Tablayoutexp **/
    protected $repositoryTablayoutexp;
    
    /** @var Factory\Table **/
    protected $factoryTable;

    public function __construct(
        EntityManager $entityManager, Utilities $utilities, TeknisaEnvironment $environment, Factory\Table $factoryTable
    ){
        $this->utilities = $utilities;
        $this->environment = $environment;
        $this->factoryTable = $factoryTable;
        $this->entityManager = $entityManager;
        $this->repositoryTabela = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Tabela');
        $this->repositoryTablayoutexp = $this->entityManager->getRepository(Utilities::INTEGRACAO_ENTITY_DIR . 'Tablayoutexp');
    }

    public function getTables($conditions, $page, $pageSize) {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return $this->repositoryTabela->getTables(Utilities::createQueryFiltered($conditions), $limit, $offset);
    }

    public function saveTable($tableName, $tableActive, $tableDescription, $tableSequential = null){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $dateNow     = new \DateTime();
            $orgId       = $this->environment->getCurrentOrganizationId();
            $codOperador = $this->environment->getCurrentUserId();

            /** @var Geral\Entities\Tabela $table */
            $table      = $this->repositoryTabela->findOneBy(array('nmtabelabd' => $tableName));
            $savedTable = $this->factoryTable->save($table, $codOperador, $dateNow, $orgId, $tableName,
                $tableDescription, $tableActive, $tableSequential);
            $this->entityManager->persist($savedTable);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }

    public function deleteTable($tableName){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $table      = $this->repositoryTabela->findOneBy(array('nmtabelabd' => $tableName));
            if ($table){
                $layoutsWithThisTable = $this->repositoryTablayoutexp->findBy(array('nmtabelabd'=> $tableName));
                if ($layoutsWithThisTable){
                    foreach ($layoutsWithThisTable as $layout){
                        $this->entityManager->remove($layout);
                    }
                    $this->entityManager->flush();
                }
                $this->entityManager->remove($table);
                $this->entityManager->flush();
                $connection->commit();
            } else {
                throw new \Exception(Messages::ERR_NOT_REMOVED_EXPLANATION ."não existe a tabela infomada!");
            }
        } catch (\Exception $ex){
            $connection->rollBack();
            throw $ex;
        }
    }

    public function getAllTables($conditions, $page, $pageSize) {
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return $this->repositoryTabela->getAllTables(Utilities::createQueryFiltered($conditions), $limit, $offset);
    }
}